/****************************************************************************
 AIAddinManager.h
 
 Copyright (c) Microsoft Corporation
 
 Shared manager for interaction with the AddinsFramework
 
 *****************************************************************************/

#import <Foundation/Foundation.h>
#import "AIApiHandler.h"
#import "AICacheDelegate.h"
#import "AICommandApp.h"
#import "AIProvider.h"
#import "AITaskpaneProvider.h"

// These guids are used for testing with hard coded manifests. They will be
// removed once we have dynamic loading.

//appCommandBase.xml
static const NSString * _Nonnull appCommandGuid = @"117062B1-1AF3-48BC-8AE3-BB25CAB9D4CA";
//AcompliSubject.xml
static const NSString * _Nonnull acompliSubjectGuid = @"dfecccda-910e-497a-bc5b-d2cc7cce728e";

/**
 * Singleton manager for interacting with the Office Solution Framework.
 */
@interface AIAddinManager : NSObject
{
	// Allow the host to specify whether we should allow enabling or not
	BOOL _addinsEnabled;
}

#pragma mark - Properties

/**
 * This delegate is called on each API call
 */
@property (retain) id<AIApiHandler> _Nullable apiDelegate;

/**
 * App commands can request a taskpane for displaying data. This delegate 
 * provides a way to call into the app to request a taskpane.
 */
@property (retain) id<AITaskpaneProvider> _Nullable taskpaneProviderDelegate;


/**
 *  When this is provided, the AddinsFramework will use the cache delegate to
 *  cache any data, rather than everything being fetched on demand. 
 *  Examples are things like the icons for command buttons, etc.
 *  If this is set, no caching will be done by the AddinsFramework at all. This
 *  should be set immediately after initialization in order to ensure that data
 *  is not cached in memory unecessarily.
 */
@property (retain) id<AICacheDelegate> _Nullable cacheDelegate;


#pragma mark - Methods

/**
 * A shared instance of the AIAddinManager class
 *
 * @return A shared instance of the AIAddinManager class
 */
+ (instancetype _Nonnull)shared;

/**
 * Initializes the OSF framework, making it ready to start loading add-ins
 *
 * @param apiDelegate               Delegate to call for API calls
 * @param taskpaneProviderDelegate  Delegate to call for taskpane manipulation
 * @param error                     The error parameter to populate on failure
 *
 * @return YES if the framework initalizes successfully, NO otherwises
 */
- (BOOL)initializeFrameworkWithApiDelegate:(id<AIApiHandler> _Nonnull)apiDelegate
			  withTaskpaneProviderDelegate:(id<AITaskpaneProvider> _Nonnull)taskpaneProviderDelegate
									 error:(NSError * _Nullable * _Nullable)error;

/**
 * Shuts down the OSF framework. Should be called on app shutdown.
 */
- (void)shutdownFramework;

/**
 * Checks if the framework is initialized
 *
 * @return YES if the framework is initialized, NO otherwise
 */
- (BOOL)isFrameworkInitialized;

/**
 *  Returns a flag stating whether addins are currently enabled
 *
 *  Designed to allow the host to specify that they shouldn't allow add-ins
 *
 *  @return true if add-ins are enabled, false otherwise
 */
- (BOOL)addinsEnabled;

/**
 *  Set whether add-ins are enabled or not
 *
 *  @param enabled Set this to true if add-ins should be enabled, false otherwise
 */
- (void)setAddinsEnabled:(BOOL)enabled;

/**
 *  Registers a provider with the OSF framework
 *
 *  @param provider The provider to register
 *
 *  @return YES on successful registration, NO otherwise
 */
- (BOOL)registerProvider:(id<AIProvider> _Nonnull)provider;

/**
 * Inserts the application with the given appId into the given container
 *
 * @param appId		The id of the add-in (taken from the manifest)
 * @param version   The version of the add-in
 * @param container The UIView to place the add-in into
 *
 * @return 	A pointer to the ControlContainer (a C++ object) created for the 
 *			add-in
 */
- (void * _Nullable)insertApp:(NSString * _Nonnull)appId atVersion:(NSString * _Nonnull)version withContainer:(UIView * _Nonnull)container;

/**
 * Unloads the add-in from the supplied ControlContainer
 *
 * @param container The ControlContainer which contains a running add-in
 */
- (void)unloadAppFromContainer:(UIView * _Nonnull)container;

/**
 * Informs the add-in that it's parent container was resized so that it can
 * update accordinly
 *
 * @param container The parent container which was resized
 */
- (void)resizedContainer:(UIView * _Nonnull)container;

/**
 * Generates and returns an with the loaded app commands
 *
 * @return A array representing the loaded app commands
 */
- (NSArray<AICommandApp *> * _Nonnull)getAppCommands;

/**
 *  Returns the doc cookie (a unique identifier) for the current document
 *
 *  @return The current doc cookie
 */
- (long)currentDocCookie;

@end
