/****************************************************************************
 AIApiHandler.h
 
 Copyright (c) Microsoft Corporation
 
 Defines an interface for the AddinsFramework to call into the host app for APIs
 
 *****************************************************************************/

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import "AIMessage.h"
#import "AINotification.h"
#import "AIApiErrorCode.h"
#import "AISandboxErrorCode.h"
#import "AITokenType.h"

/**
 *  A protocol to be implemented for the Addin framework to call APIs.
 *
 *  Some APIs will shared methods, however, others will each have their own
 *  method. Each API will only act on the currently displayed message, or cause
 *  some action to be taken on the currently displayed message, or in the
 *  current context (e.g. create a new calendar event)
 */
@protocol AIApiHandler <NSObject>

/**
 *  The current message which is being displayed to the user.
 *
 *  @param error An error parameter to be populated on failure
 *
 *  @return The current message
 */
- (id<AIMessage> _Nullable)currentMessage:(NSError * _Nullable * _Nullable)error;

/**
 *  The account for the current mailbox/message
 *
 *  @param error An error parameter to be populated on failure
 *
 *  @return The current account
 */
- (id<AIAccount> _Nullable)currentAccount:(NSError * _Nullable * _Nullable)error;

/**
 *  Returns the number of currently displaying notifications
 *
 *  @return The number of currently displaying notifications
 */
- (int)numberOfDisplayedNotificationsForDocCookie:(NSNumber * _Nonnull)docCookie;

/**
 *  Display a notifification to the user
 *
 *  @param message The notification to display
 *
 *  @return The relevant error code
 */
- (AIApiErrorCode)displayNotification:(AINotification * _Nonnull)notification;

/**
 *  Gets the notifications current displayed to the user for a particular message
 *
 *  @param docCookie The doc cookie of the message we want the notifications for
 *
 *  @return An array of AINotifications currently displayed to the user
 */
- (NSArray<AINotification*> * _Nullable)notificationsForDocCookie:(NSNumber * _Nonnull)docCookie;

/**
 *  Remove the notification on screen with the supplied key
 *
 *  @param docCookie The doc cookie for the message that the notification is on
 *
 *  @return The relevant error code
 */
- (AIApiErrorCode)removeNotificationForDocCookie:(NSNumber * _Nonnull)docCookie
										 withKey:(NSString * _Nonnull)key;

/**
 *  Informs the host to display the default progress notification (if no other
 *  non-persistent notification is showing)
 * 
 *  @param docCookie The doc cookie for the message to display the notification 
 *                   on
 */
- (void)displayDefaultProgressNotificationForDocCookie:(NSNumber * _Nonnull)docCookie;

/**
 *  Informs the host to remove the default progress notification (if showing)
 *
 *  @param docCookie The doc cookie for the message which has the notification
 */
- (void)removeDefaultProgressNotificationForDocCookie:(NSNumber * _Nonnull)docCookie;

/**
 *  Requests a token from Exchange
 *
 *  @param tokenType   The type of token being requested
 *  @param account     The account the token is being requested for
 *  @param extensionId The ID of the extension to get a token for
 *  @param completion  Block called on completion of getting a token
 */
- (void)getToken:(AITokenType)tokenType
	  forAccount:(id<AIAccount> _Nonnull)account
 withExtensionId:(NSString * _Nonnull)extensionId
	onCompletion:(void (^ _Nonnull)(AIApiErrorCode errorCode, NSString * _Nullable token))completion;

/**
 *  Gets the roaming settings for an extension and its particular version
 *
 *  @param extensionId      The ID of the extension
 *  @param extensionVersion The version of the extension that is requesting 
 *                          the settings
 *  @param account          The account to store the settings on
 *  @param completion       The block to call on completion of getting the 
 *							settings
 */
- (void)getExtensionSettingsForExtensionId:(NSString * _Nonnull)extensionId
						  extensionVersion:(NSString * _Nonnull)extensionVersion
								   account:(id<AIAccount> _Nonnull)account
								completion:(void (^ _Nonnull)(AIApiErrorCode errorCode, NSString * _Nullable settings))completion;

/**
 *  Sets roaming settings for an extension and its particular version
 *
 *  @param settings         A JSON string representing the settings that comes
 *                          from the JS layer
 *  @param extensionId      The ID of the extension which is setting the 
 *                          settings
 *  @param extensionVersion The version of the extension which is setting the
 *                          settings
 *  @param account          The account to store the settings on
 *  @param completion       The block to call on completion of setting the 
 *                          settings (This should probably be _Nullable, but it
 *                          causes problems with the integartion into Swift code
 *                          if it is).
 */
- (void)setExtensionSettings:(NSString * _Nonnull)settings
			  forExtensionId:(NSString * _Nonnull)extensionId
			extensionVersion:(NSString * _Nonnull)extensionVersion
					 account:(id<AIAccount> _Nonnull)account
				  completion:(void (^ _Nonnull)(AIApiErrorCode errorCode))completion;

/**
 *  Gets the properties for an extension and its particular version
 *
 *  @param extensionId      The ID of the extension
 *  @param extensionVersion The version of the extension that is requesting
 *                          the properties
 *  @param messageId		The exchange ID of the message to get the properties
 *                          from
 *  @param account          The account to store the properties on
 *  @param completion       The block to call on completion of getting the
 *							properties
 */
- (void)getExtensionPropertiesForExtensionId:(NSString * _Nonnull)extensionId
							extensionVersion:(NSString * _Nonnull)extensionVersion
								   messageId:(NSString * _Nonnull)messageId
									 account:(id<AIAccount> _Nonnull)account
								  completion:(void (^ _Nonnull)(AIApiErrorCode errorCode, NSString * _Nullable properties))completion;

/**
 *  Sets properties for an extension and its particular version
 *
 *  @param properties       A JSON string representing the properties that comes
 *                          from the JS layer
 *  @param extensionId      The ID of the extension which is setting the
 *                          properties
 *  @param extensionVersion The version of the extension which is setting the
 *                          properties
 *  @param messageId        The exchange ID of the message to store the 
 *                          properties on
 *  @param account          The account to store the properties on
 *  @param completion       The block to call on completion of setting the
 *                          properties (This should probably be _Nullable, but 
 *							it causes problems with the integartion into Swift 
 *							code if it is).
 */
- (void)setExtensionProperties:(NSString * _Nonnull)properties
				forExtensionId:(NSString * _Nonnull)extensionId
			  extensionVersion:(NSString * _Nonnull)extensionVersion
					 messageId:(NSString * _Nonnull)messageId
					   account:(id<AIAccount> _Nonnull)account
					completion:(void (^ _Nonnull)(AIApiErrorCode errorCode))completion;


/**
 *  Requests that the host handles the display of an error
 *
 *  @param errorCode  The general error code for the error
 *  @param view       The container view that the error has occured on. If the 
 *                    view is nil, it means that the error occured in a UILess
 *                    app command
 *  @param statusCode The exact status code of the error
 *
 *  @return true if the error was handled, false otherwise
 */
- (bool)handleError:(AISandboxErrorCode)errorCode onView:(UIView * _Nullable)view statusCode:(int)statusCode;

@end

