/****************************************************************************
 AICommandButton.h
 
 Copyright (c) Microsoft Corporation
 
 Stores the information for an app command button
 
 *****************************************************************************/

#import "AIGraphicalElement.h"
#import <UIKit/UIKit.h>

@interface AICommandButton : AIGraphicalElement

/**
 *  The name of the addin that the button belongs to
 */
@property (copy) NSString * _Nullable addinName;

/**
 *  The unique identifier of the button which is used to look up the native
 *  implementation
 */
@property (copy) NSNumber * _Nullable rawReference;

/**
 *  The ID of the extension point this button belongs to
 */
@property (copy) NSString * _Nullable extensionPointId;

/**
 *  The solution ID that this button belongs to
 */
@property (copy) NSString * _Nullable solutionId;

/**
 *  The tab ID that this button belongs to
 */
@property (copy) NSString * _Nullable tabId;

/**
 *  Creates a button by looking up a native instance using the supplied 
 *  parameters
 *
 *  @param extensionPointId The ID of the extension point the button belongs to
 *  @param elementId        The ID of the element the button belongs to
 *  @param solutionId       The ID of the solution the button belongs to
 *  @param tabId            The ID of the tab the button belongs to
 *
 *  @return A new button created from the supplied parameters.
 */
+ (instancetype _Nullable)buttonWithExtensionPointId:(NSString * _Nullable)extensionPointId
										   elementId:(NSString * _Nullable)elementId
										  solutionId:(NSString * _Nullable)solutionId
											   tabId:(NSString * _Nullable)tabId;

/**
 *  Creates a button using the supplied parameters
 *
 *  @param identifier       The unique identifier for the button
 *  @param ownerId          The ID of the button owner
 *  @param label            The label to be displayed on a graphical 
 *                          representation of the button
 *  @param addinName        The name of the addin the button belongs to
 *  @param extensionPointId The ID of the extension point the button belongs to
 *  @param solutionId       The ID of the solution the button belongs to
 *  @param tabId            The ID of the tab the button belongs to
 *  @param rawReference     A unique identifier for the native implementation
 *                          which backs this button
 *
 *  @return A new button created from the supplied parameters
 */
- (instancetype _Nonnull)initWithIdentifier:(NSString * _Nullable)identifier
									ownerId:(NSString * _Nullable)ownerId
									  label:(NSString * _Nullable)label
								  addinName:(NSString * _Nullable)addinName
						   extensionPointId:(NSString * _Nullable)extensionPointId
								 solutionId:(NSString * _Nullable)solutionId
									  tabId:(NSString * _Nullable)tabId
							   rawReference:(NSNumber * _Nullable)rawReference;

/**
 *  Gets the URL of the icon for the app that this button belongs to
 *
 *  @return A URL for the app icon
 */
- (NSURL * _Nullable)appIconURL;

/**
 *  Gets the URL of the icon for a graphical representation of the button which
 *  most closely matches the supplied size. 
 *
 *  If the size supplied matches one defined exactly, that URL will be returned.
 *  If it is not matched exactly, then the next larger size will be returned. If
 *  there is not a larger size, the URL of the largest available icon will be 
 *  returned.
 *
 *  @param size The preferred size of the icon (square)
 *
 *  @return The best matching icon URL for the size
 */
- (NSURL * _Nullable)iconURLForSize:(CGFloat)size;

/**
 *  Indicates that a user has pressed the button
 */
- (void)press;

@end
