/****************************************************************************
 AIMessage.h
 
 Copyright (c) Microsoft Corporation
 
 Defines the interface for a message
 
 *****************************************************************************/

#import <Foundation/Foundation.h>
#import "AIAccount.h"
#import "AIAttachment.h"
#import "AIContact.h"
#import "AIMeetingRequest.h"
#import "AIMessageType.h"

/**
 * All information about a message that may be required by the Addin
 * framework.
 *
 * @see <a href="https://dev.outlook.com/reference/add-ins/Office.context.mailbox.item.html#to">Office.context.mailbox.item</a>
 */
@protocol AIMessage <NSObject>

/**
 * The exchange ID for the message
 *
 * @return The exchange ID for the message
 */
- (NSString * _Nonnull)exchangeID;

/**
 * The message id as defined by RFC 2392
 *
 * @return The message id of the email
 *
 * @see https://tools.ietf.org/html/rfc2392
 */
- (NSString * _Nonnull)internetMessageID;

/**
 * A unique ID for the conversation that this message is a part of
 *
 * @return A unique ID for the conversation that this message is a part of
 */
- (NSString * _Nonnull)conversationID;

/**
 * The subject line of the email
 *
 * @return The subject line of the email
 */
- (NSString * _Nonnull)subject;

/**
 * A normalized verison of the email subject as defined by (TODO dalemy *Insert reference here*)
 *
 * @return The normalized email subject
 */
- (NSString * _Nonnull)normalizedSubject;

/**
 * The trimmed body of the email, whether plaintext or HTML
 *
 * The trimmed body may be equivalent to the full body
 *
 * @return The trimmed body of the email
 */
- (NSString * _Nonnull)trimmedBody;

/**
 * The full body of the email, whether plaintext or HTML
 *
 * @return The full body of the email
 */
- (NSString * _Nullable)fullBody;

/**
 * An array of the attachments in this email
 *
 * @return An array of attachments conforming to AIAttachment
 */
- (NSArray<id<AIAttachment>> * _Nullable)attachments;

/**
 * The unix timestamp of the emails creation, in milliseconds
 *
 * @return The unix timestamp fo the emails creation, in milliseconds
 */
- (NSDate * _Nonnull)dateTimeCreated;

/**
 * The unix timestamp of the last modification of the email, in milliseconds
 *
 * @return The unix timestamp of the last modification of the email, in milliseconds
 */
- (NSDate * _Nonnull)dateTimeModified;

/**
 * The unix timestamp of when the email was sent, in milliseconds
 *
 * @return The unix timestamp of when the email was sent, in milliseconds
 */
- (NSDate * _Nonnull)dateTimeSent;

/**
 * The class of the item
 *
 * @return The class of the item
 */
- (NSString * _Nullable)itemClassName;

/**
 * The contact that this email is from
 *
 * @return The contact that this email is from
 */
- (id<AIContact> _Nullable)fromContact;

/**
 * The contact that this email was sent by
 *
 * This differs from the fromContact in the case where someone sends an email
 * on another's behalf. In this case, the sender of the email is here, whereas 
 * the person who the email was sent on the behalf of, is the fromContact.
 *
 * @return The contact that this email was sent by
 */
- (id<AIContact> _Nullable)senderContact;

/**
 * The set of contacts that this email was sent to
 *
 * @return The set of contacts that this email was sent to
 */
- (NSSet<id<AIContact>> * _Nullable)toContacts;

/**
 * The set of contacts that this email was CC'd to
 *
 * @return The set of contacts that this email was CC'd to
 */
- (NSSet<id<AIContact>> * _Nullable)ccContacts;

/**
 * The set of contacts that this email was BCC'd to
 *
 * @return The set of contacts that this email was BCC'd to
 */
- (NSSet<id<AIContact>> * _Nullable)bccContacts;

/**
 * The account that the message belongs to
 *
 * @return The account that the message belongs to
 */
- (id<AIAccount> _Nonnull)account;

/**
 *  The type of the message
 *
 *  For example, is it a regular mail, appointment, etc.
 *
 *  @return The type of the message
 */
- (AIMessageType)messageType;

/**
 *  Checks if the message is HTML or text
 *
 *  @return YES if the message is HTML, NO otherwise
 */
- (BOOL)isHTMLMessage;

/**
 *  If the message has a meeting requestm, return it here
 *
 *  @return A meeting request, if the message has one
 */
- (id<AIMeetingRequest> _Nullable)meetingRequest;

@end

