/****************************************************************************
 AINotification.h
 
 Copyright (c) Microsoft Corporation
 
 Represents a notification
 
 *****************************************************************************/

#import <Foundation/Foundation.h>
#import "AINotificationType.h"
#import "AIStore.h"
#import "AIMessage.h"

@interface AINotification : NSObject

/**
 *  The unique key for this notification
 */
@property (retain) NSString * _Nonnull key;

/**
 *  The actual text message which should be displayed to the user
 */
@property (retain) NSString * _Nonnull message;

/**
 *  The name of the icon resource to display. Nil if the default should be used
 */
@property (retain) NSString * _Nullable iconResourceName;

/**
 *  The URL of the icon to display. Nil if the default should be used
 */
@property (retain) NSURL * _Nullable iconURL;

/**
 *  If tye type is informational, then this flag may be set to ensure that this
 *  message continues to show, even on changing email
 */
@property BOOL isPersistent;

/**
 *  Set to true if this notification should replace an existing one
 */
@property BOOL isReplacement;

/**
 *  The type of notification this is (informational, progress or error)
 */
@property AINotificationType type;

/**
 *  Unique identifier for the document this notification is associated with
 */
@property (retain) NSNumber * _Nonnull docCookie;

/**
 *  ID of the add-in which has requested the notification
 */
@property (retain) NSString * _Nonnull appId;

/**
 *  A flag for whether this is a system notification or not (and therefore 
 *  exempt from any maximum notification rules). This defaults to false.
 */
@property BOOL isSystemNotification;

/**
 *  Creates a new AINotification
 *
 *  @param key              The unique key for the notification
 *  @param message          The text to dislpay to the user
 *  @param iconResourceName The resource id of the icon (if there is one)
 *  @param iconURL          The URL of the icon (if there is one)
 *  @param persistent       YES if the notification should persist, NO otherwise
 *  @param replacement      YES if this should replace another notification with
 *							the same key (should it exist), NO otherwise
 *  @param type             The type of the notification (informational, 
 *							progress, or error)
 *  @param appId            The ID of the add-in which has sent the notification
 *  @param docCookie        The doc cookie of the message to show the 
 *							notification on.
 *
 *  @return A new AINotification
 */
- (instancetype _Nonnull)initWithKey:(NSString * _Nonnull)key
							 message:(NSString * _Nonnull)message
					iconResourceName:(NSString * _Nullable)iconResourceName
							 iconURL:(NSURL * _Nullable)iconURL
						  persistent:(BOOL)persistent
						 replacement:(BOOL)replacement
								type:(AINotificationType)type
							   appId:(NSString * _Nonnull)appId
						   docCookie:(NSNumber * _Nonnull)docCookie;


/**
 *  Gets a dictionary of the exchange properties of this AINotification
 *
 *  @return A dictionary of the Exchange properties
 */
- (NSDictionary<NSString *, NSObject *> * _Nonnull)exchangeProperties;


/**
 *  Creates an array of AINotifications from the Exchange notification
 *  data.
 *
 *  @param json			The raw JSON string
 *  @param docCookie	The document cookie that these notifications are for
 *  @param store		The store the notification came from (in case we test 
 *						with the developer store).
 *
 *  @return An array of AINotification on success, nil on parse failure
 */
+ (NSArray<AINotification *> * _Nullable)notificationsFromJSON:(NSString * _Nonnull)json
													 docCookie:(NSNumber * _Nonnull)docCookie
														 store:(AIStore)store;

/**
 *  Converts a list of notifications (from the same extension) to the JSON 
 *  required for sending to Exchange for updates.
 *
 *  @param notifications	An array of notifications from the same extension to
 *							create the JSON for
 *  @param appId			The ID of the extension we are going to be updating
 *							the notifications for
 *
 *  @return The JSON representation
 */
+ (NSString * _Nullable)jsonForNotifications:(NSArray<AINotification *> * _Nonnull)notifications
								   withAppId:(NSString * _Nonnull)appId;

@end
